<?php

/**
 * @file
 * Contains Drupal\masquerade\Masquerade.
 */

namespace Drupal\masquerade;

use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Routing\LinkGeneratorTrait;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\user\UserInterface;
use Psr\Log\LoggerInterface;

/**
 * Defines a masquerade service to switch user account.
 */
class Masquerade {
  use StringTranslationTrait;
  use LinkGeneratorTrait;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The session manager.
   *
   * @var \Drupal\Core\Session\SessionManagerInterface
   */
  protected $sessionManager;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  public function __construct(AccountInterface $current_user, EntityManagerInterface $entity_manager, ModuleHandlerInterface $module_handler, SessionManagerInterface $session_manager, LoggerInterface $logger) {
    $this->currentUser = $current_user;
    $this->entityManager = $entity_manager;
    $this->moduleHandler = $module_handler;
    $this->sessionManager = $session_manager;
    $this->logger = $logger;
  }

  /**
   * Returns whether the current user is masquerading.
   *
   * @return bool
   */
  public function isMasquerading() {
    // @todo Check to use some session related service.
    return !empty($_SESSION['masquerading']);
  }

  /**
   * Masquerades the current user as a given user.
   *
   * @param \Drupal\user\UserInterface $target_account
   *   The user account object to masquerade as.
   *
   * @return bool
   *   TRUE when masqueraded, FALSE otherwise.
   */
  public function switchTo(UserInterface $target_account) {
    $account = $this->currentUser;

    // Call logout hooks when switching from original user.
    $this->moduleHandler->invokeAll('user_logout', [$account]);

    // Regenerate the session ID to prevent against session fixation attacks.
    $this->sessionManager->regenerate();

    $_SESSION['masquerading'] = $account->id();

    // Supposed "safe" user switch method:
    // https://www.drupal.org/node/218104
    //$accountSwitcher = Drupal::service('account_switcher');
    //$accountSwitcher->switchTo(new UserSession(array('uid' => $account->id())));
    $this->currentUser->setAccount($target_account);
    \Drupal::service('session')->set('uid', $target_account->id());

    // Call all login hooks when switching to masquerading user.
    $this->moduleHandler->invokeAll('user_login', [$target_account]);

    $this->logger->info('User %username masqueraded as %target_username.', array(
      '%username' => $account->getUsername(),
      '%target_username' => $target_account->getUsername(),
      'link' => $this->l($this->t('view'), $target_account->urlInfo()),
    ));
    return TRUE;
  }

  /**
   * Switching back to previous user.
   *
   * @return bool
   *   TRUE when switched back, FALSE otherwise.
   */
  public function switchBack() {
    if (empty($_SESSION['masquerading'])) {
      return FALSE;
    }
    $new_user = $this->entityManager
      ->getStorage('user')
      ->load($_SESSION['masquerading']);

    // Ensure the flag is cleared.
    unset($_SESSION['masquerading']);
    if (!$new_user) {
      return FALSE;
    }

    $account = $this->currentUser;
    // Call logout hooks when switching from masquerading user.
    $this->moduleHandler->invokeAll('user_logout', [$account]);
    // Regenerate the session ID to prevent against session fixation attacks.
    // @todo Maybe session service migrate.
    $this->sessionManager->regenerate();

    $this->currentUser->setAccount($new_user);
    \Drupal::service('session')->set('uid', $new_user->id());

    // Call all login hooks when switching back to original user.
    $this->moduleHandler->invokeAll('user_login', [$new_user]);

    $this->logger->info('User %username stopped masquerading as %old_username.', array(
      '%username' => $new_user->getUsername(),
      '%old_username' => $account->getUsername(),
      'link' => $this->l($this->t('view'), $new_user->urlInfo()),
    ));
    return TRUE;
  }

}
