<?php

/**
 * @file
 * Contains \Drupal\masquerade\Tests\MasqueradeAccessTest.
 */

namespace Drupal\masquerade\Tests;

use Drupal\user\UserInterface;
use Drupal\Core\Session\UserSession;

/**
 * Tests masquerade access mechanism.
 *
 * @todo Convert into DUTB. This is essentially a unit test for
 *   masquerade_target_user_access() only.
 *
 * @group masquerade
 */
class MasqueradeAccessTest extends MasqueradeWebTestBase {

  /**
   * Tests masquerade access for different source and target users.
   *
   * Test plan summary:
   * - root » admin
   * - admin ! root
   * - admin » moderator (more roles but less privileges)
   * - admin » masquerade (different role)
   * - admin » auth (less roles)
   * - moderator ! root
   * - moderator ! admin (less roles but more privileges)
   * - moderator ! editor (different roles + privileges)
   * - moderator » masquerade (less roles)
   * - moderator » auth
   * - [editor is access-logic-wise equal to moderator, so skipped]
   * - masquerade ! root
   * - masquerade ! admin (different role with more privileges)
   * - masquerade ! moderator (more roles)
   * - masquerade » auth
   * - masquerade ! masquerade (self)
   * - auth ! *
   */
  function testAccess() {
    $this->drupalLogin($this->root_user);
    $this->assertCanMasqueradeAs($this->admin_user);

    $this->drupalLogin($this->admin_user);
    $this->assertCanNotMasqueradeAs($this->root_user);
    $this->assertCanMasqueradeAs($this->moderator_user);
    $this->assertCanMasqueradeAs($this->masquerade_user);
    $this->assertCanMasqueradeAs($this->auth_user);

    $this->drupalLogin($this->moderator_user);
    $this->assertCanNotMasqueradeAs($this->root_user);
    $this->assertCanNotMasqueradeAs($this->admin_user);
    $this->assertCanNotMasqueradeAs($this->editor_user);
    $this->assertCanMasqueradeAs($this->masquerade_user);
    $this->assertCanMasqueradeAs($this->auth_user);

    $this->drupalLogin($this->masquerade_user);
    $this->assertCanNotMasqueradeAs($this->root_user);
    $this->assertCanNotMasqueradeAs($this->admin_user);
    $this->assertCanNotMasqueradeAs($this->moderator_user);
    $this->assertCanMasqueradeAs($this->auth_user);

    // Verify that a user cannot masquerade as himself.
    $edit = array(
      'masquerade_as' => $this->masquerade_user->getUsername(),
    );
    $this->drupalPostForm('masquerade', $edit, t('Switch'));
    $this->assertRaw(t('You cannot masquerade as yourself. Please choose a different user to masquerade as.'));
    $this->assertNoText(t('Unmasquerade'));

    // Basic 'masquerade' permission check.
    $this->drupalLogin($this->auth_user);
    $this->drupalGet('masquerade');
    $this->assertResponse(403);
  }

  /**
   * Asserts that the currently logged-in user can masquerade as a given target user.
   *
   * @param \Drupal\user\UserInterface|\Drupal\Core\Session\UserSession $target_account
   *   The user to masquerade or session of root_user.
   */
  protected function assertCanMasqueradeAs($target_account) {
    $edit = array(
      'masquerade_as' => $target_account->getUsername(),
    );
    $this->drupalPostForm('masquerade', $edit, t('Switch'));
    $this->assertNoRaw(t('You are not allowed to masquerade as %name.', array(
      '%name' => $target_account->getUsername(),
    )));
    $this->clickLink(t('Unmasquerade'));
  }

  /**
   * Asserts that the currently logged-in user can not masquerade as a given target user.
   *
   * @param \Drupal\user\UserInterface|\Drupal\Core\Session\UserSession $target_account
   *   The user to masquerade or session of root_user.
   */
  protected function assertCanNotMasqueradeAs($target_account) {
    $edit = array(
      'masquerade_as' => $target_account->getUsername(),
    );
    $this->drupalPostForm('masquerade', $edit, t('Switch'));
    $this->assertRaw(t('You are not allowed to masquerade as %name.', array(
      '%name' => $target_account->getUsername(),
    )));
    $this->assertNoText(t('Unmasquerade'));
  }

}
